/*
  *****************************************************************************

  Copyright 1993 The Regents of the University of California.
  All Rights Reserved.

  Permission to use, copy, modify and distribute this software and its
  documentation for educational, research and non-profit purposes, without
  fee, and without a written agreement is hereby granted, provided that the 
  above copyright notice and the following three paragraphs appear in all 
  copies.

  Permission to incorporate this software into commercial products may be
  obtained from the Office of Technology Licensing, 2150 Shattuck Avenue,
  Suite 510, Berkeley, CA 94720-1620.

  IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
  LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
  OF SUCH DAMAGES.

  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
  ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS
  TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

  The author may be contacted via:

  US Mail:  Brian Mirtich
            387 Soda Hall
	    Computer Science Division
	    Department of Electrical Engineering and Computer Science
	    University of California
	    Berkeley, CA 94720

 Phone:     (510) 642-8149

 EMail:     mirtich@cs.berkeley.edu

 *****************************************************************************
*/



#ifndef MATVECT_HEADER
#define MATVECT_HEADER

/*
  =============================================================================
  specify single or double precision
  =============================================================================
*/

typedef double REAL;


/*
  =============================================================================
  constants
  =============================================================================
*/

#define DEG_TO_RAD (M_PI / 180.0)
#define RAD_TO_DEG (180.0 / M_PI)


/*
  =============================================================================
  data structures
  =============================================================================
*/

typedef REAL MAT3[3][3];
typedef REAL MAT4[4][4];

typedef REAL VECT3[3];
typedef REAL VECT4[4];


/*
  =============================================================================
  macros
  =============================================================================
*/

/* include the following macro if your math library doesn't have sincos */
#define sincos(angle, s, c) { *(s) = sin(angle); *(c) = cos(angle); } 


/*
  =============================================================================
  global variables from matvect.c
  =============================================================================
*/

extern MAT4 mat4I;


/*
  =============================================================================
  functions from matvect.c
  =============================================================================
*/

void mat4copy();
void mat4mult();
void matMultXform();
void matInvXform();
void matBuildXform();

void vectCopy();
void vectSwap();
void vectAdd();
void vectSub();
void vectNeg();
void vectScale();
void vectNormalize();
REAL vectNorm();
int vectEqual();
REAL vectDotProd();
void vectXprod();

void xformPoint();
void xformVect();
void xform3();

REAL planeDist();
void displacePoint();




#endif
